@@ -1,14 +1,13 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+/* A Bison parser, made by GNU Bison 3.0.4.  */
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,9 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -33,49 +30,45 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-/* Tokens.  */
+#ifndef YY_YY_HTMLPARSE_H_INCLUDED
+# define YY_YY_HTMLPARSE_H_INCLUDED
+/* Debug traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 1
+#endif
+#if YYDEBUG
+extern int yydebug;
+#endif
+
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     T_WAIT = 258,
-     T_ERROR = 259,
-     T_TEXT = 260,
-     T_ELEMENT_START = 261,
-     T_ELEMENT_START_END = 262,
-     T_ELEMENT_END = 263,
-     T_SCRIPT = 264,
-     T_STYLE = 265,
-     T_PI = 266,
-     T_COMMENT = 267,
-     T_CDATA = 268,
-     T_DOCTYPE = 269
-   };
+  enum yytokentype
+  {
+    T_WAIT = 258,
+    T_ERROR = 259,
+    T_TEXT = 260,
+    T_ELEMENT_START = 261,
+    T_ELEMENT_START_END = 262,
+    T_ELEMENT_END = 263,
+    T_SCRIPT = 264,
+    T_STYLE = 265,
+    T_PI = 266,
+    T_COMMENT = 267,
+    T_CDATA = 268,
+    T_DOCTYPE = 269
+  };
 #endif
-/* Tokens.  */
-#define T_WAIT 258
-#define T_ERROR 259
-#define T_TEXT 260
-#define T_ELEMENT_START 261
-#define T_ELEMENT_START_END 262
-#define T_ELEMENT_END 263
-#define T_SCRIPT 264
-#define T_STYLE 265
-#define T_PI 266
-#define T_COMMENT 267
-#define T_CDATA 268
-#define T_DOCTYPE 269
-
-
-
 
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef int YYSTYPE;
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
-# define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
+# define YYSTYPE_IS_DECLARED 1
 #endif
 
 
 
+int yyparse (PyObject* scanner);
+
+#endif /* !YY_YY_HTMLPARSE_H_INCLUDED  */
