@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2008 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,29 +11,35 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 A blacklist logger.
 """
 
 import os
-
-import linkcheck.logger
+import codecs
+from linkcheck.configuration import get_user_data
+from . import _Logger
 
 
-class BlacklistLogger (linkcheck.logger.Logger):
+class BlacklistLogger (_Logger):
     """
     Updates a blacklist of wrong links. If a link on the blacklist
     is working (again), it is removed from the list. So after n days
     we have only links on the list which failed for n days.
     """
 
-    def __init__ (self, **args):
-        """
-        Intialize with old blacklist data (if found, else not).
-        """
+    LoggerName = "blacklist"
+
+    LoggerArgs = {
+        "filename": os.path.join(get_user_data(), "blacklist"),
+    }
+
+    def __init__ (self, **kwargs):
+        """Intialize with old blacklist data (if found, else not)."""
+        args = self.get_args(kwargs)
         super(BlacklistLogger, self).__init__(**args)
         self.init_fileoutput(args)
         self.blacklist = {}
@@ -50,18 +56,18 @@ class BlacklistLogger (linkcheck.logger.Logger):
         """
         Put invalid url in blacklist, delete valid url from blacklist.
         """
-        if not url_data.cached:
-            key = url_data.cache_url_key
-            if key in self.blacklist:
-                if url_data.valid:
-                    del self.blacklist[key]
-                else:
-                    self.blacklist[key] += 1
+        key = (url_data.parent_url, url_data.cache_url)
+        key = repr(key)
+        if key in self.blacklist:
+            if url_data.valid:
+                del self.blacklist[key]
             else:
-                if not url_data.valid:
-                    self.blacklist[key] = 1
+                self.blacklist[key] += 1
+        else:
+            if not url_data.valid:
+                self.blacklist[key] = 1
 
-    def end_output (self):
+    def end_output (self, **kwargs):
         """
         Write blacklist file.
         """
@@ -71,24 +77,22 @@ class BlacklistLogger (linkcheck.logger.Logger):
         """
         Read a previously stored blacklist from file fd.
         """
-        fd = open(self.filename)
-        try:
+        with codecs.open(self.filename, 'r', self.output_encoding,
+                         self.codec_errors) as fd:
             for line in fd:
-                line = self.decode(line.rstrip())
+                line = line.rstrip()
                 if line.startswith('#') or not line:
                     continue
                 value, key = line.split(None, 1)
                 self.blacklist[key] = int(value)
-        finally:
-            fd.close()
 
     def write_blacklist (self):
         """
         Write the blacklist.
         """
         oldmask = os.umask(0077)
-        for key, value in self.blacklist.iteritems():
-            self.write(u"%d %s%s" % (value, key, os.linesep))
+        for key, value in self.blacklist.items():
+            self.write(u"%d %s%s" % (value, repr(key), os.linesep))
         self.close_fileoutput()
         # restore umask
         os.umask(oldmask)
