@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2008 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,9 +11,9 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Some functions have been taken and adjusted from the quodlibet
 # source. Quodlibet is (C) 2004-2005 Joe Wreschnig, Michael Urman
@@ -22,6 +22,10 @@
 Various string utility functions. Note that these functions are not
 necessarily optimised for large strings, so use with care.
 """
+# some handy time constants
+SECONDS_PER_MINUTE = 60
+SECONDS_PER_HOUR = 60 * SECONDS_PER_MINUTE
+SECONDS_PER_DAY = 24 * SECONDS_PER_HOUR
 
 import re
 import textwrap
@@ -29,13 +33,18 @@ import codecs
 import os
 import math
 import time
-import urlparse
+try:
+    import urlparse
+except ImportError:
+    # Python 3
+    from urllib import parse as urlparse
+import locale
 import pydoc
+from . import i18n
 
 
-def unicode_safe (s, encoding="iso-8859-1"):
-    """
-    Get unicode string without raising encoding errors. Unknown
+def unicode_safe (s, encoding=i18n.default_encoding, errors='replace'):
+    """Get unicode string without raising encoding errors. Unknown
     characters of the given encoding will be ignored.
 
     @param s: the string to be decoded
@@ -48,12 +57,11 @@ def unicode_safe (s, encoding="iso-8859-1"):
     if isinstance(s, unicode):
         # s is already unicode, nothing to do
         return s
-    return unicode(str(s), encoding, "ignore")
+    return unicode(str(s), encoding, errors)
 
 
 def ascii_safe (s):
-    """
-    Get ASCII string without raising encoding errors. Unknown
+    """Get ASCII string without raising encoding errors. Unknown
     characters of the given encoding will be ignored.
 
     @param s: the Unicode string to be encoded
@@ -67,9 +75,7 @@ def ascii_safe (s):
 
 
 def is_ascii (s):
-    """
-    Test if a string can be encoded in ASCII.
-    """
+    """Test if a string can be encoded in ASCII."""
     try:
         s.encode('ascii', 'strict')
         return True
@@ -78,9 +84,7 @@ def is_ascii (s):
 
 
 def is_encoding (text):
-    """
-    Check if string is a valid encoding.
-    """
+    """Check if string is a valid encoding."""
     try:
         return codecs.lookup(text)
     except (LookupError, ValueError):
@@ -88,15 +92,12 @@ def is_encoding (text):
 
 
 def url_unicode_split (url):
-    """
-    Like urlparse.urlsplit(), but always returning unicode parts.
-    """
+    """Like urlparse.urlsplit(), but always returning unicode parts."""
     return [unicode_safe(s) for s in urlparse.urlsplit(url)]
 
 
 def unquote (s, matching=False):
-    """
-    Remove leading and ending single and double quotes.
+    """Remove leading and ending single and double quotes.
     The quotes need to match if matching is True. Only one quote from each
     end will be stripped.
 
@@ -125,44 +126,37 @@ _para_win = r"(?:%(sep)s)(?:(?:%(sep)s)\s*)+" % {'sep': '\r\n'}
 _para_ro = re.compile("%s|%s|%s" % (_para_mac, _para_posix, _para_win))
 
 def get_paragraphs (text):
-    """
-    A new paragraph is considered to start at a line which follows
+    """A new paragraph is considered to start at a line which follows
     one or more blank lines (lines containing nothing or just spaces).
-    The first line of the text also starts a paragraph.
-    """
+    The first line of the text also starts a paragraph."""
     if not text:
         return []
     return _para_ro.split(text)
 
 
 def wrap (text, width, **kwargs):
-    """
-    Adjust lines of text to be not longer than width. The text will be
+    """Adjust lines of text to be not longer than width. The text will be
     returned unmodified if width <= 0.
     See textwrap.wrap() for a list of supported kwargs.
-    Returns text with lines no longer than given width.
-    """
+    Returns text with lines no longer than given width."""
     if width <= 0 or not text:
         return text
     ret = []
     for para in get_paragraphs(text):
-        ret.extend(textwrap.wrap(para.strip(), width, **kwargs))
+        text = " ".join(para.strip().split())
+        ret.extend(textwrap.wrap(text, width, **kwargs))
     return os.linesep.join(ret)
 
 
 def indent (text, indent_string="  "):
-    """
-    Indent each line of text with the given indent string.
-    """
+    """Indent each line of text with the given indent string."""
     lines = str(text).splitlines()
-    return os.linesep.join(["%s%s" % (indent_string, x) for x in lines])
+    return os.linesep.join("%s%s" % (indent_string, x) for x in lines)
 
 
 def get_line_number (s, index):
-    r"""
-    Return the line number of s[index] or zero on errors.
-    Lines are assumed to be separated by the ASCII character '\n'.
-    """
+    r"""Return the line number of s[index] or zero on errors.
+    Lines are assumed to be separated by the ASCII character '\n'."""
     i = 0
     if index < 0:
         return 0
@@ -175,18 +169,14 @@ def get_line_number (s, index):
 
 
 def paginate (text):
-    """
-    Print text in pages of lines.
-    """
+    """Print text in pages of lines."""
     pydoc.pager(text)
 
 
 _markup_re = re.compile("<.*?>", re.DOTALL)
 
 def remove_markup (s):
-    """
-    Remove all <*> html markup tags from s.
-    """
+    """Remove all <*> html markup tags from s."""
     mo = _markup_re.search(s)
     while mo:
         s = s[0:mo.start()] + s[mo.end():]
@@ -194,34 +184,29 @@ def remove_markup (s):
     return s
 
 
-def strsize (b):
-    """
-    Return human representation of bytes b. A negative number of bytes
-    raises a value error.
-    """
+def strsize (b, grouping=True):
+    """Return human representation of bytes b. A negative number of bytes
+    raises a value error."""
     if b < 0:
         raise ValueError("Invalid negative byte number")
     if b < 1024:
-        return u"%dB" % b
+        return u"%sB" % locale.format("%d", b, grouping)
     if b < 1024 * 10:
-        return u"%dKB" % int(b / 1024)
+        return u"%sKB" % locale.format("%d", (b // 1024), grouping)
     if b < 1024 * 1024:
-        return u"%.2fKB" % (float(b) / 1024)
+        return u"%sKB" % locale.format("%.2f", (float(b) / 1024), grouping)
     if b < 1024 * 1024 * 10:
-        return u"%.2fMB" % (float(b) / (1024*1024))
+        return u"%sMB" % locale.format("%.2f", (float(b) / (1024*1024)), grouping)
     if b < 1024 * 1024 * 1024:
-        return u"%.1fMB" % (float(b) / (1024*1024))
+        return u"%sMB" % locale.format("%.1f", (float(b) / (1024*1024)), grouping)
     if b < 1024 * 1024 * 1024 * 10:
-        return u"%.2fGB" % (float(b) / (1024*1024*1024))
-    return u"%.1fGB" % (float(b) / (1024*1024*1024))
+        return u"%sGB" % locale.format("%.2f", (float(b) / (1024*1024*1024)), grouping)
+    return u"%sGB" % locale.format("%.1f", (float(b) / (1024*1024*1024)), grouping)
 
 
-def strtime (t):
-    """
-    Return ISO 8601 formatted time.
-    """
-    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(t)) + \
-           strtimezone()
+def strtime (t, func=time.localtime):
+    """Return ISO 8601 formatted time."""
+    return time.strftime("%Y-%m-%d %H:%M:%S", func(t)) + strtimezone()
 
 
 # from quodlibet
@@ -233,18 +218,27 @@ def strduration (duration):
     else:
         prefix = ""
     duration = math.ceil(duration)
-    if duration >= 3600: # 1 hour
+    if duration >= SECONDS_PER_HOUR: # 1 hour
         # time, in hours:minutes:seconds
-        return "%s%02d:%02d:%02d" % (prefix, duration // 3600,
-                                   (duration % 3600) // 60, duration % 60)
+        return "%s%02d:%02d:%02d" % (prefix, duration // SECONDS_PER_HOUR,
+                                   (duration % SECONDS_PER_HOUR) // SECONDS_PER_MINUTE,
+                                    duration % SECONDS_PER_MINUTE)
     else:
         # time, in minutes:seconds
-        return "%s%02d:%02d" % (prefix, duration // 60, duration % 60)
+        return "%s%02d:%02d" % (prefix, duration // SECONDS_PER_MINUTE,
+                                 duration % SECONDS_PER_MINUTE)
 
 
 # from quodlibet
-def strduration_long (duration):
+def strduration_long (duration, do_translate=True):
     """Turn a time value in seconds into x hours, x minutes, etc."""
+    if do_translate:
+        # use global translator functions
+        global _, _n
+    else:
+        # do not translate
+        _ = lambda x: x
+        _n = lambda a, b, n: a if n==1 else b
     if duration < 0:
         duration = abs(duration)
         prefix = "-"
@@ -283,29 +277,25 @@ def strduration_long (duration):
 
 
 def strtimezone ():
-    """
-    Return timezone info, %z on some platforms, but not supported on all.
+    """Return timezone info, %z on some platforms, but not supported on all.
     """
     if time.daylight:
         zone = time.altzone
     else:
         zone = time.timezone
-    return "%+04d" % int(-zone/3600)
+    return "%+04d" % (-zone//SECONDS_PER_HOUR)
 
 
-_sub_ws = re.compile(r"\s+").sub
-def stripall (s):
-    """
-    Remove all whitespace from given string.
-    """
+def stripurl(s):
+    """Remove any lines from string after the first line.
+    Also remove whitespace at start and end from given string."""
     if not s:
         return s
-    return _sub_ws("", s)
+    return s.splitlines()[0].strip()
 
 
 def limit (s, length=72):
-    """
-    If the length of the string exceeds the given limit, it will be cut
+    """If the length of the string exceeds the given limit, it will be cut
     off and three dots will be appended.
 
     @param s: the string to limit
@@ -320,3 +310,22 @@ def limit (s, length=72):
     if length == 0:
         return ""
     return "%s..." % s[:length]
+
+
+def strline (s):
+    """Display string representation on one line."""
+    return strip_control_chars(u"`%s'" % unicode(s).replace(u"\n", u"\\n"))
+
+
+def format_feature_warning (**kwargs):
+    """Format warning that a module could not be imported and that it should
+    be installed for a certain URL.
+    """
+    return _("Could not import %(module)s for %(feature)s. Install %(module)s from %(url)s to use this feature.") % kwargs
+
+
+def strip_control_chars(text):
+    """Remove console control characters from text."""
+    if text:
+        return re.sub(r"[\x01-\x1F\x7F]", "", text)
+    return text
